@@ -51,10 +51,11 @@ makedist <- function(z, data, distancefn, within = NULL) {
     
     if ((nc * nr > getMaxProblemSize()) && warning.requested) {
 
-      warning("I've been asked to compute a large number of treatment-control distances. 
-The result will present too large an optimization problem for optimal matching.  
-You can split up or simplify the problem by providing an appropriate 'within'
-argument; see 'match_on', 'exactMatch' and 'caliper' documentation for details.")  
+      warning("match_on has been asked to compute a large number of treatment-control
+distances.  You can reduce this number by providing an appropriate 'within'
+argument; see 'match_on', 'exactMatch' and 'caliper' documentation for
+details.  Alternatively you could increase the problem size limit; see
+documentation of 'getMaxProblemSize' and 'setMaxProblemSize'.")  
 
     }
 
@@ -85,7 +86,9 @@ elaborating on it; see 'exactMatch' and 'caliper' documentation for details.")
     }
   }
 
-  dists <- distancefn(cbind(treatmentids, controlids), data, z)
+    dists <- distancefn(cbind(treatmentids, controlids), data, z)
+    if (length(dists)!=length(treatmentids))
+        stop("Internal optmatch error - malformed distances.\n (Try a different method argument to match_on.)")
 
   # z was copied <- toZ(z) so should be safe to rm
   # before massive matrix alloc
